//
//  EAGLView.h
//  Checkerboard
//
//  Created by vorg on 2009-03-12.
//  Copyright Vorg, http://www.vorg.pl 2009. All rights reserved.
//


#import <UIKit/UIKit.h>
#import <OpenGLES/EAGL.h>
#import <OpenGLES/ES1/gl.h>
#import <OpenGLES/ES1/glext.h>
#import <AVFoundation/AVAudioPlayer.h>

/*
This class wraps the CAEAGLLayer from CoreAnimation into a convenient UIView subclass.
The view content is basically an EAGL surface you render your OpenGL scene into.
Note that setting the view non-opaque will only work if the EAGL surface has an alpha channel.
*/
@interface EAGLView : UIView {
    
@private
    /* The pixel dimensions of the backbuffer */
    GLint backingWidth;
    GLint backingHeight;
    
    EAGLContext *context;
	AVAudioPlayer *player;
	
	CFAbsoluteTime lastTime;
	CFTimeInterval frameTime;
    
    /* OpenGL names for the renderbuffer and framebuffers used to render to this view */
    GLuint viewRenderbuffer, viewFramebuffer;
    
    /* OpenGL name for the depth buffer that is attached to viewFramebuffer, if it exists (0 if it does not exist) */
    GLuint depthRenderbuffer;
	
	GLuint checkerTex;
	GLuint dmaTex;	
	GLuint scrollColorsTex; 
	GLuint rainbowTex;
	GLuint fakeFogTex;
	GLuint starsTex;
	GLuint greetingsTex;	
	GLuint troLogo;	
	GLuint dmaPresentsTex;
	GLuint compoContributionTex;

	CGFloat angle;
	CGFloat time;
    
    NSTimer *animationTimer;
    NSTimeInterval animationInterval;
}

@property NSTimeInterval animationInterval;
@property (nonatomic,retain) AVAudioPlayer* player;

- (void)startAnimation;
- (void)stopAnimation;
- (void)setupAudio;
- (void)drawView;
- (void)drawFloor;
- (void)drawDmaPresents;
- (void)drawTroLogo;
- (void)drawDmaScroll;
- (void)drawGreetingsScroll;
- (void)drawCube;
- (void)captureToPhotoAlbum;
- (UIImage *) glToUIImage;

@end
