//
//  EAGLView.m
//  Checkerboard
//
//  Created by vorg on 2009-03-12.
//  Copyright Vorg, http://www.vorg.pl 2009. All rights reserved.
//



#import <OpenGLES/EAGLDrawable.h>


#import "EAGLView.h"
#import "myglu.h"

#define DEGREES_TO_RADIANS(__ANGLE__) ((__ANGLE__) / 180.0 * M_PI)

GLfloat travelTime = 18;
GLfloat rainbowTime = 12;
GLfloat greetingsStart = 30;

// A class extension to declare private methods
@interface EAGLView ()

@property (nonatomic, retain) EAGLContext *context;
@property (nonatomic, assign) NSTimer *animationTimer;

- (BOOL) createFramebuffer;
- (void) destroyFramebuffer;

@end

@interface EAGLView (EAGLViewSprite)

- (void)setupView;

@end

@interface EAGLView (AVAudioPlayerDelegate)

- (void)audioPlayerDidFinishPlaying:(AVAudioPlayer *)player successfully:(BOOL)flag;

@end


@implementation EAGLView

@synthesize context;
@synthesize animationTimer;
@synthesize animationInterval;
@synthesize player;


// You must implement this method
+ (Class)layerClass {
    return [CAEAGLLayer class];
}


//The GL view is stored in the nib file. When it's unarchived it's sent -initWithCoder:
- (id)initWithCoder:(NSCoder*)coder {
    
    if ((self = [super initWithCoder:coder])) {
        // Get the layer
        CAEAGLLayer *eaglLayer = (CAEAGLLayer *)self.layer;
        
        eaglLayer.opaque = YES;
        eaglLayer.drawableProperties = [NSDictionary dictionaryWithObjectsAndKeys:
                                        [NSNumber numberWithBool:NO], kEAGLDrawablePropertyRetainedBacking, kEAGLColorFormatRGBA8, kEAGLDrawablePropertyColorFormat, nil];
        
        context = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES1];
        
        if (!context || ![EAGLContext setCurrentContext:context]) {
            [self release];
            return nil;
        }
        
		animationInterval = 1.0 / 30.0;

		
		[self setupView];
		[self setupAudio];
		
		lastTime = CFAbsoluteTimeGetCurrent();
		//[self drawView];        
    }
    return self;
}

- (void)setupView {
	perspective(90, 3.0f/4.0f, 0.1, 100);
	
	checkerTex = loadTexture(@"checkerboard_16_256.png");
	dmaTex = loadTexture(@"dma_logo_iphone_invert.png");
	scrollColorsTex = loadTexture(@"scroll_colors.png");
	rainbowTex = loadTexture(@"rainbow.png");
	fakeFogTex = loadTexture(@"fakeFog.png");
	starsTex = loadTexture(@"stars.png");
	greetingsTex = loadTexture(@"greetingz.png");
	troLogo = loadTexture(@"troLogoColor.png");
	dmaPresentsTex = loadTexture(@"dma_presents.png");
	compoContributionTex = loadTexture(@"compo_contribution.png");	
	
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);	
	
	angle = 0;
	time = 0;
}

- (void)setupAudio {
	NSString *soundFilePath = [[NSBundle mainBundle] pathForResource: @"soundtrack_little_ninja_girl_theme" ofType: @"wav"];
	
	NSURL *fileURL = [[NSURL alloc] initFileURLWithPath: soundFilePath];
	
	AVAudioPlayer *newPlayer = [[AVAudioPlayer alloc] initWithContentsOfURL: fileURL error: nil];
	[fileURL release];
	
	self.player = newPlayer;
	[newPlayer release];
	
	[self.player prepareToPlay];
	[self.player setDelegate: self];
	[self.player play];
}

- (void)drawView {
	[EAGLContext setCurrentContext:context];
	
	CFAbsoluteTime currTime = CFAbsoluteTimeGetCurrent();
	frameTime = currTime - lastTime;
	lastTime = currTime;
	
	frameTime = 1/30.0f;
	time += frameTime;
	angle += 30*frameTime;   
	
	glBindFramebufferOES(GL_FRAMEBUFFER_OES, viewFramebuffer);
	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	perspective(90, 3.0f/4.0f, 0.1, 100);
	
	glMatrixMode(GL_MODELVIEW);
    glViewport(0, 0, backingWidth, backingHeight);
	glLoadIdentity();   
		
	GLfloat light1_ambient[] = { 0.2, 0.2, 0.2, 1.0 };
	GLfloat light1_diffuse[] = { 1.0, 0.0, 1.0, 1.0 };
	glLightfv(GL_LIGHT1, GL_AMBIENT, light1_ambient);
	glLightfv(GL_LIGHT1, GL_DIFFUSE, light1_diffuse);
	GLfloat lightPos[] = {5.0, 5.0, 5.0, 0.0};	
	glLightfv(GL_LIGHT1, GL_POSITION, lightPos);
	glEnable(GL_LIGHT1);
	//glShadeModel(GL_FLAT);

	
	glRotatef(-90, 0, 0, 1);	
	lookAt(0, 0, -0.2, 0, 0, -15, 0, 1, 0);	
	
	[self drawFloor];
	[self drawDmaPresents];
	[self drawTroLogo];	
	[self drawDmaScroll];	
	[self drawGreetingsScroll];	
	[self drawCube];
	
	glBindRenderbufferOES(GL_RENDERBUFFER_OES, viewRenderbuffer);
    [context presentRenderbuffer:GL_RENDERBUFFER_OES];
	
	//screen capture
	//[self captureToPhotoAlbum];	
}

- (void)drawFloor {
	const GLfloat floorSize = 3.7f;
    const GLfloat floorVertices[] = {
		-floorSize, 0, -floorSize,
        floorSize,  0, -floorSize,
        -floorSize, 0, floorSize,
        floorSize,  0, floorSize
    };
	
	const GLubyte floorColors[] = {
        255, 255, 255, 200,
        255,  255, 255, 200,
        255, 255, 255, 200,
        255, 255, 255, 200
    };
	
	const GLfloat floorTexcoords[] = {
		0, 0 - time/5,
		1, 0 - time/5,
		0, 1 - time/5,
		1, 1 - time/5
	};
	
	const GLfloat fakeFogVertices[] = {
		0,  80,
        480, 80,
        0,  240,
        480, 240
    };
	
	const GLshort fakeFogTexcoords[] = {
		0, 0,
		1, 0,
		0, 1,
		1, 1
	};
	
	const GLfloat bgVertices[] = {
		0,  0,
        480, 0,
        0,  160,
        480, 160
    };
	
   	const GLfloat bgTexcoords[] = {
		0, 0 - angle/40.0f,
		1, 0 - angle/40.0f,
		0, 1 - angle/40.0f,
		1, 1 - angle/40.0f
	};
	
	const GLfloat bgTexcoords2[] = {
		0, 0 + angle/40.0f,
		1, 0 + angle/40.0f,
		0, 1 + angle/40.0f,
		1, 1 + angle/40.0f
	};
	
	const GLfloat rainbowTransitionTime = 5;
	GLubyte whiteAlha = 255;
	if (time > rainbowTime) {
		if (time - rainbowTime < rainbowTransitionTime) {
			whiteAlha = 255 - 255*((time - rainbowTime)/rainbowTransitionTime);
		}
		else {
			whiteAlha = 0;
		}
	}
	const GLubyte whiteColors[] = {
        255, 255, 255, whiteAlha,
        255,  255, 255, whiteAlha,
        255, 255, 255, whiteAlha,
        255, 255, 255, whiteAlha
    };
	
	const GLfloat whiteVertices[] = {
		0,  0,
        480, 0,
        0,  320,
        480, 320
    };
	
	//rainbow
	
	startOrtho();	
		glPushMatrix();
			glDisable(GL_BLEND);
			glEnableClientState(GL_VERTEX_ARRAY);
			glVertexPointer(2, GL_FLOAT, 0, bgVertices);
			
			glEnable(GL_TEXTURE_2D);
			glBindTexture(GL_TEXTURE_2D, rainbowTex);		
			glEnableClientState(GL_TEXTURE_COORD_ARRAY);	
			glTexCoordPointer(2, GL_FLOAT, 0, bgTexcoords);	
			
			glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);	
			
			glTranslatef(0, 160, 0);
			glTexCoordPointer(2, GL_FLOAT, 0, bgTexcoords2);	
			glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);	
		glPopMatrix();
		
		//white
		
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glDisable(GL_TEXTURE_2D);
		glEnableClientState(GL_VERTEX_ARRAY);
		glVertexPointer(2, GL_FLOAT, 0, whiteVertices);
		glEnableClientState(GL_COLOR_ARRAY);		
		glColorPointer(4, GL_UNSIGNED_BYTE, 0, whiteColors);		
		glDisableClientState(GL_TEXTURE_COORD_ARRAY);		
		glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);		
		
		glDisableClientState(GL_COLOR_ARRAY);
	endOrtho();
	
    
	glEnable(GL_BLEND);
	glBlendFunc(GL_ZERO, GL_ONE_MINUS_SRC_COLOR);
	
	//checker
	
	glDepthMask(GL_FALSE);
	glDisable(GL_DEPTH_TEST);
	
	glPushMatrix();				
		if (time > travelTime) {
			glRotatef(30*travelTime-angle, 0.0f, 1.0f, 0.0f);
		}
	
		glVertexPointer(3, GL_FLOAT, 0, floorVertices);
		glEnableClientState(GL_VERTEX_ARRAY);
		glColorPointer(4, GL_UNSIGNED_BYTE, 0, floorColors);
		glEnableClientState(GL_COLOR_ARRAY);
		
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, checkerTex);		
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);	
		glTexCoordPointer(2, GL_FLOAT, 0, floorTexcoords);
		
		glTranslatef(0, -0.62f, 0);
		glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
		
		glTranslatef(0, 1.24f, 0);	
		glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);	
		glPopMatrix();
	
	glDisableClientState(GL_COLOR_ARRAY);
	
	glEnable(GL_BLEND);
	glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
	
	glDepthMask(GL_TRUE);
	glEnable(GL_DEPTH_TEST);

	//shadow
	
	startOrtho();
		glEnableClientState(GL_VERTEX_ARRAY);
		glVertexPointer(2, GL_FLOAT, 0, fakeFogVertices);
		
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, fakeFogTex);		
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);	
		glTexCoordPointer(2, GL_SHORT, 0, fakeFogTexcoords);	
		
		glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);	
	endOrtho();
	
	glDisable(GL_BLEND);
	
	}

- (void)drawDmaPresents {
	const GLfloat textVertices[] = {
		(480-256)/2,     (320-32)/2,
        (480-256)/2+256, (320-32)/2,
        (480-256)/2,     (320-32)/2+32,
        (480-256)/2+256, (320-32)/2+32
    };
	
   	const GLfloat textTexcoords[] = {
		0, 0,
		1, 0,
		0, 1,
		1, 1
	};
	
	GLubyte textAlpha = 255;
	
	glEnable(GL_TEXTURE_2D);
	if (time < 1) {
		return;
	}
	if (time < 5) {
		glBindTexture(GL_TEXTURE_2D, dmaPresentsTex);	
		if (time < 2) textAlpha = 255 * (time-1);
		else if (time > 4) textAlpha = 255 - 255 * (time-4);
	}
	else if (time < 9) {
		glBindTexture(GL_TEXTURE_2D, compoContributionTex);
		if (time < 6) textAlpha = 255 * (time-5);
		else if (time > 8) textAlpha = 255 - 255 * (time-8);
	}
	else { //>9
		return;
	}
	
	const GLubyte textColors[] = {
        255, 255, 255, textAlpha,
        255,  255, 255, textAlpha,
        255, 255, 255, textAlpha,
        255, 255, 255, textAlpha
    };
	
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
	glPushMatrix();
	startOrtho();
					
		glEnableClientState(GL_VERTEX_ARRAY);
		glVertexPointer(2, GL_FLOAT, 0, textVertices);
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
		glTexCoordPointer(2, GL_FLOAT, 0, textTexcoords);	
		glEnableClientState(GL_COLOR_ARRAY);		
		glColorPointer(4, GL_UNSIGNED_BYTE, 0, textColors);			
		glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);	
		glDisable(GL_TEXTURE_2D);	
	endOrtho();	
	glPopMatrix();
	
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);		
}


- (void)drawTroLogo {
	
	float y = 0;
	
	if (time >= 9 && time < 10) {
		y = -200 + bounceEaseOut(time - 9, 0, 200, 1);
	}
	const GLfloat textVertices[] = {
		(480-256)/2,     (320-128)/2 + y,
        (480-256)/2+256, (320-128)/2 + y,
        (480-256)/2,     (320-128)/2+128 + y,
        (480-256)/2+256, (320-128)/2+128 + y
    };
	
   	const GLfloat textTexcoords[] = {
		0, 0,
		1, 0,
		0, 1,
		1, 1
	};
	
	GLubyte textAlpha = 255;
	
	glEnable(GL_TEXTURE_2D);
	if (time < 9) {
		return;
	}
	if (time < travelTime) {
		glBindTexture(GL_TEXTURE_2D, troLogo);	
		if (time > travelTime-1) textAlpha = 255 - 255 * (time-travelTime+1);
	}
	else {
		return;
	}
	
	
	const GLubyte textColors[] = {
        255, 255, 255, textAlpha,
        255,  255, 255, textAlpha,
        255, 255, 255, textAlpha,
        255, 255, 255, textAlpha
    };
	
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
	glPushMatrix();
	startOrtho();
					
		glEnableClientState(GL_VERTEX_ARRAY);
		glVertexPointer(2, GL_FLOAT, 0, textVertices);
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
		glTexCoordPointer(2, GL_FLOAT, 0, textTexcoords);	
		glEnableClientState(GL_COLOR_ARRAY);		
		glColorPointer(4, GL_UNSIGNED_BYTE, 0, textColors);			
		glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);	
		glDisable(GL_TEXTURE_2D);	
	endOrtho();	
	glPopMatrix();
	
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
}

- (void)drawDmaScroll {
	const GLfloat scrollAmplitude = 0.3;
	const GLfloat dmaVertices[] = {
		-8, -0.7 + scrollAmplitude*sin(angle/10 + M_PI*0/4.0), 0,		
		-8,  0.7 + scrollAmplitude*sin(angle/10 + M_PI*0/4.0), 0,
	 	-6, -0.7 + scrollAmplitude*sin(angle/10 + M_PI*1/4.0), 0,
		-6,  0.7 + scrollAmplitude*sin(angle/10 + M_PI*1/4.0), 0,
		-4, -0.7 + scrollAmplitude*sin(angle/10 + M_PI*2/4.0), 0,
		-4,  0.7 + scrollAmplitude*sin(angle/10 + M_PI*2/4.0), 0,
		-2, -0.7 + scrollAmplitude*sin(angle/10 + M_PI*3/4.0), 0,
		-2,  0.7 + scrollAmplitude*sin(angle/10 + M_PI*3/4.0), 0,
		0, -0.7 + scrollAmplitude*sin(angle/10 + M_PI*4/4.0), 0,		
		0,  0.7 + scrollAmplitude*sin(angle/10 + M_PI*4/4.0), 0,		
		2, -0.7 + scrollAmplitude*sin(angle/10 + M_PI*5/4.0), 0,
		2,  0.7 + scrollAmplitude*sin(angle/10 + M_PI*5/4.0), 0,
		4, -0.7 + scrollAmplitude*sin(angle/10 + M_PI*6/4.0), 0,
		4,  0.7 + scrollAmplitude*sin(angle/10 + M_PI*6/4.0), 0,
		6, -0.7 + scrollAmplitude*sin(angle/10 + M_PI*7/4.0), 0,
		6,  0.7 + scrollAmplitude*sin(angle/10 + M_PI*7/4.0), 0,
		8, -0.7 + scrollAmplitude*sin(angle/10 + M_PI*8/4.0), 0,		
		8,  0.7 + scrollAmplitude*sin(angle/10 + M_PI*8/4.0), 0	
	};
	
	const GLfloat slideTime = 5;
	const GLfloat scrollTime = time - travelTime;
	const GLfloat texShift = (scrollTime < slideTime) ? 0 : (scrollTime - slideTime)/1.2;
	const GLfloat posShift = (scrollTime < slideTime) ? 12-12*scrollTime/slideTime : 0;
	const GLfloat dmaTexcoords[] = {
		0.7*0.0f + texShift, 1.0f,		
		0.7*0.0f + texShift, 0.0f,
		0.7*1.0f + texShift, 1.0f,
		0.7*1.0f + texShift, 0.0f,	
		0.7*2.0f + texShift, 1.0f,
		0.7*2.0f + texShift, 0.0f,	
		0.7*3.0f + texShift, 1.0f,
		0.7*3.0f + texShift, 0.0f,
		0.7*4.0f + texShift, 1.0f,
		0.7*4.0f + texShift, 0.0f,		
		0.7*5.0f + texShift, 1.0f,
		0.7*5.0f + texShift, 0.0f,	
		0.7*6.0f + texShift, 1.0f,
		0.7*6.0f + texShift, 0.0f,	
		0.7*7.0f + texShift, 1.0f,
		0.7*7.0f + texShift, 0.0f,
		0.7*8.0f + texShift, 1.0f,
		0.7*8.0f + texShift, 0.0f
	};	
	
	const GLubyte dmaReflectionColors[] = {
		255, 255, 255, 50,		
		255, 255, 255, 50,
	 	255, 255, 255, 50,
		255, 255, 255, 50,
		255, 255, 255, 50,
		255, 255, 255, 50,
		255, 255, 255, 50,
		255, 255, 255, 50,
		255, 255, 255, 50,		
		255, 255, 255, 50,		
		255, 255, 255, 50,
		255, 255, 255, 50,
		255, 255, 255, 50,
		255, 255, 255, 50,
		255, 255, 255, 50,
		255, 255, 255, 50,
		255, 255, 255, 50,		
		255, 255, 255, 50	
	};
	
	const GLfloat scrollColorsVertices[] = {
		-8, -0.9, 0, 
		-8,  0.9, 0,
		8, -0.9, 0,		
		8,  0.9, 0		
	};
	
	
	const GLfloat scrollColorsTexcoords[] = {
		0.0f + texShift, 0.0f,		
		0.0f + texShift, 1.0f,		
		1.0f + texShift, 0.0f,
		1.0f + texShift, 1.0f
	};	
	
	const GLubyte starsColors[] = {
		255, 255, 255, 255*(0.5 + 0.5*sin(angle/10 + M_PI*0/4.0)),		
		255, 255, 255, 255*(0.5 + 0.5*sin(angle/10 + M_PI*0/4.0)),
	 	255, 255, 255, 255*(0.5 + 0.5*sin(angle/10 + M_PI*1/4.0)),
		255, 255, 255, 255*(0.5 + 0.5*sin(angle/10 + M_PI*1/4.0)),
		255, 255, 255, 255*(0.5 + 0.5*sin(angle/10 + M_PI*2/4.0)),
		255, 255, 255, 255*(0.5 + 0.5*sin(angle/10 + M_PI*2/4.0)),
		255, 255, 255, 255*(0.5 + 0.5*sin(angle/10 + M_PI*3/4.0)),
		255, 255, 255, 255*(0.5 + 0.5*sin(angle/10 + M_PI*3/4.0)),
		255, 255, 255, 255*(0.5 + 0.5*sin(angle/10 + M_PI*4/4.0)),		
		255, 255, 255, 255*(0.5 + 0.5*sin(angle/10 + M_PI*4/4.0)),		
		255, 255, 255, 255*(0.5 + 0.5*sin(angle/10 + M_PI*5/4.0)),
		255, 255, 255, 255*(0.5 + 0.5*sin(angle/10 + M_PI*5/4.0)),
		255, 255, 255, 255*(0.5 + 0.5*sin(angle/10 + M_PI*6/4.0)),
		255, 255, 255, 255*(0.5 + 0.5*sin(angle/10 + M_PI*6/4.0)),
		255, 255, 255, 255*(0.5 + 0.5*sin(angle/10 + M_PI*7/4.0)),
		255, 255, 255, 255*(0.5 + 0.5*sin(angle/10 + M_PI*7/4.0)),
		255, 255, 255, 255*(0.5 + 0.5*sin(angle/10 + M_PI*8/4.0)),		
		255, 255, 255, 255*(0.5 + 0.5*sin(angle/10 + M_PI*8/4.0))
	};
	
	glDisable(GL_DEPTH_TEST);
	glEnable(GL_BLEND);
	glEnable(GL_TEXTURE_2D);
	glPushMatrix();			
		glTranslatef(posShift, 0, -5);	
		glDisableClientState(GL_COLOR_ARRAY);
	
		glBindTexture(GL_TEXTURE_2D, dmaTex);			
		glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);			
		glVertexPointer(3, GL_FLOAT, 0, dmaVertices);
		glEnableClientState(GL_VERTEX_ARRAY);		
		glTexCoordPointer(2, GL_FLOAT, 0, dmaTexcoords);
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);	
		glDrawArrays(GL_TRIANGLE_STRIP, 0, 18);
	
		glBindTexture(GL_TEXTURE_2D, scrollColorsTex);
		glBlendFunc(GL_ZERO, GL_SRC_COLOR);
		glVertexPointer(3, GL_FLOAT, 0, scrollColorsVertices);
		glEnableClientState(GL_VERTEX_ARRAY);
		glTexCoordPointer(2, GL_FLOAT, 0, scrollColorsTexcoords);
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);		
		glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
	
		glBindTexture(GL_TEXTURE_2D, starsTex);			
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);			
		glVertexPointer(3, GL_FLOAT, 0, dmaVertices);
		glEnableClientState(GL_VERTEX_ARRAY);
		glColorPointer(4, GL_UNSIGNED_BYTE, 0, starsColors);
		glEnableClientState(GL_COLOR_ARRAY);
		glTexCoordPointer(2, GL_FLOAT, 0, dmaTexcoords);
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);	
		glDrawArrays(GL_TRIANGLE_STRIP, 0, 18);	 
			
	glPopMatrix();		
	
	
	
	glPushMatrix();	
		glScalef(1,-1, 1);
		glTranslatef(posShift, 2.3f, -5);	
		
		glBindTexture(GL_TEXTURE_2D, dmaTex);	
		
		glBlendFunc(GL_SRC_ALPHA, GL_ONE);		
		glVertexPointer(3, GL_FLOAT, 0, dmaVertices);
		glEnableClientState(GL_VERTEX_ARRAY);		
		glColorPointer(4, GL_UNSIGNED_BYTE, 0, dmaReflectionColors);
		glEnableClientState(GL_COLOR_ARRAY);
		glTexCoordPointer(2, GL_FLOAT, 0, dmaTexcoords);
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);						
		
		glDrawArrays(GL_TRIANGLE_STRIP, 0, 18);				
	glPopMatrix();	
	
	glDisableClientState(GL_COLOR_ARRAY);
	glDisable(GL_BLEND);
	glEnable(GL_DEPTH_TEST);
}

- (void)drawGreetingsScroll {	
	const GLfloat greetingsVertices[] = {
		0,  0,
        1024, 0,
        0,  32,
        1024, 32
    };
	
	const GLshort greetingsTexcoords[] = {
		0, 0,
		1, 0,
		0, 1,
		1, 1
	};	
	
	glDisable(GL_DEPTH_TEST);
	glEnable(GL_BLEND);
	glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
	
	glPushMatrix();		
	startOrtho();
		glTranslatef(480 + 64*greetingsStart - 64*time, 270, 0);
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, greetingsTex);		
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);	
		glTexCoordPointer(2, GL_SHORT, 0, greetingsTexcoords);	
		glEnableClientState(GL_VERTEX_ARRAY);
		glVertexPointer(2, GL_FLOAT, 0, greetingsVertices);
		glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);	
	endOrtho();	
	glPopMatrix();	
}

- (void)drawCube {
	return;
	const GLfloat faceVertices[] = {
		-1, -1, 1,
		 1, -1, 1,
		-1,  1, 1,
		 1, -1, 1,
		-1,  1, 1,
		 1,  1, 1
	};
	
	const GLfloat faceNormals[] = {
		-1, -1, 1,
		 1, -1, 1,
		-1,  1, 1,
		 1, -1, 1,
		-1,  1, 1,
		 1,  1, 1								
	};
	
	const GLfloat polyVertices[] = {
		//tf
		-1, 0, 1,
		 1, 0, 1,
		 0, 1, 0,
		 //tr		
		 1, 0, 1,
		 1, 0,-1,
		 0, 1, 0,
		 //tb	
		 1, 0,-1,
		-1, 0,-1,
		 0, 1, 0,
		 //tl
		-1, 0,-1,
		-1, 0, 1,		
		 0, 1, 0,
		 //bf
		-1, 0, 1,
		 1, 0, 1,
		 0,-1, 0,
		 //br		
		 1, 0, 1,
		 1, 0,-1,
		 0,-1, 0,
		 //bb	
		 1, 0,-1,
		-1, 0,-1,
		 0,-1, 0,
		 //bl
		-1, 0,-1,
		-1, 0, 1,		
		 0,-1, 0
    };
	
	
	GLfloat polyNormals[] = {
		//tf
		-1, 0, 1,
		 1, 0, 1,
		 0, 1, 0,
		 //tr		
		 1, 0, 1,
		 1, 0,-1,
		 0, 1, 0,
		 //tb	
		 1, 0,-1,
		-1, 0,-1,
		 0, 1, 0,
		 //tl
		-1, 0,-1,
		-1, 0, 1,		
		 0, 1, 0,
		 //bf
		-1, 0, 1,
		 0,-1, 0,
		 1, 0, 1,
		 //br		
		 1, 0, 1,
		 0,-1, 0,
		 1, 0,-1,		 
		 //bb	
		 1, 0,-1,
		 0,-1, 0,
		-1, 0,-1,		 
		 //bl
		-1, 0,-1,		
		 0,-1, 0,
		-1, 0, 1,
		//tf
		-1, 0, 1,
		 1, 0, 1,
		 0, 1, 0,
		 //tr		
		 1, 0, 1,
		 1, 0,-1,
		 0, 1, 0,
		 //tb	
		 1, 0,-1,
		-1, 0,-1,
		 0, 1, 0,
		 //tl
		-1, 0,-1,
		-1, 0, 1,		
		 0, 1, 0,
		 //bf
		-1, 0, 1,
		 0,-1, 0,
		 1, 0, 1,
		 //br		
		 1, 0, 1,
		 0,-1, 0,
		 1, 0,-1,		 
		 //bb	
		 1, 0,-1,
		 0,-1, 0,
		-1, 0,-1,		 
		 //bl
		-1, 0,-1,		
		 0,-1, 0,
		-1, 0, 1		 
    };
	
		
	for(int i=0; i<24*3; i+=3) {
		float nx = polyVertices[i];
		float ny = polyVertices[i+1];
		float nz = polyVertices[i+2];
		float len = sqrt(nx*nx + ny*ny + nz*nz);
		if (len == 0) len = 1;
		nx /= len;
		ny /= len;
		nz /= len;
		polyNormals[i] = nx;
		polyNormals[i+1] = ny;
		polyNormals[i+2] = nz;	
		/*
		float x0 = polyVertices[i];
		float y0 = polyVertices[i+1];
		float z0 = polyVertices[i+2];
		float x1 = polyVertices[i+3];
		float y1 = polyVertices[i+4];
		float z1 = polyVertices[i+5];
		float x2 = polyVertices[i+6];
		float y2 = polyVertices[i+7];
		float z2 = polyVertices[i+8];
		float x10 = x1 - x0;
		float y10 = y1 - y0;
		float z10 = z1 - z0;				
		float x20 = x2 - x0;
		float y20 = y2 - y0;
		float z20 = z2 - z0;
		float nx = y10*z20 - z10*y20;
		float ny = z10*x20 - x10*z20;
		float nz = x10*y20 - y10*x20;
		float len = sqrt(nx*nx + ny*ny + nz*nz);
		if (len == 0) len = 1;
		nx /= len;
		ny /= len;
		nz /= len;
		polyNormals[i] = nx;
		polyNormals[i+1] = ny;
		polyNormals[i+2] = nz;	
		polyNormals[i+3] = nx;
		polyNormals[i+4] = ny;
		polyNormals[i+5] = nz;	
		polyNormals[i+6] = nx;
		polyNormals[i+7] = ny;
		polyNormals[i+8] = nz;
		*/				
	}


	GLubyte polyAlpha = 255;
	const GLubyte polyColors[] = {
		//top
        255, 255, 255, polyAlpha,
        255,  255, 255, polyAlpha,
        255, 255, 255, polyAlpha,
		
		255, 255, 255, polyAlpha,
        255,  255, 255, polyAlpha,
        255, 255, 255, polyAlpha,
		
		255, 255, 255, polyAlpha,
        255,  255, 255, polyAlpha,
        255, 255, 255, polyAlpha,
		
		255, 255, 255, polyAlpha,
        255,  255, 255, polyAlpha,
        255, 255, 255, polyAlpha,
		//bot
		255, 255, 255, polyAlpha,
        255,  255, 255, polyAlpha,
        255, 255, 255, polyAlpha,
		
		255, 255, 255, polyAlpha,
        255,  255, 255, polyAlpha,
        255, 255, 255, polyAlpha,
		
		255, 255, 255, polyAlpha,
        255,  255, 255, polyAlpha,
        255, 255, 255, polyAlpha,
		
		255, 255, 255, polyAlpha,
        255,  255, 255, polyAlpha,
        255, 255, 255, polyAlpha
    };	
	
	glDisable(GL_TEXTURE_2D);
	glPushMatrix();				
				
		glTranslatef(0, 0, -5);	
		glRotatef(angle, 1, 1, 0);
				
		glEnable(GL_LIGHTING);		
		//glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
						
		glVertexPointer(3, GL_FLOAT, 0, faceVertices);
		glEnableClientState(GL_VERTEX_ARRAY);		
		glNormalPointer(GL_FLOAT, 0, faceNormals);
		glEnableClientState(GL_NORMAL_ARRAY);
		glColorPointer(4, GL_UNSIGNED_BYTE, 0, polyColors);
		glEnableClientState(GL_COLOR_ARRAY);
		glDisableClientState(GL_TEXTURE_COORD_ARRAY);						
		
		glPushMatrix();
			glDrawArrays(GL_TRIANGLES, 0, 6);	
			glRotatef(90, 0, 1, 0);
			glDrawArrays(GL_TRIANGLES, 0, 6);
			glRotatef(90, 0, 1, 0);
			glDrawArrays(GL_TRIANGLES, 0, 6);
			glRotatef(90, 0, 1, 0);
			glDrawArrays(GL_TRIANGLES, 0, 6);	
		glPopMatrix();
		glPushMatrix();
			glRotatef(90, 1, 0, 0);
			glDrawArrays(GL_TRIANGLES, 0, 6);
			glRotatef(180, 0, 1, 0);
			glDrawArrays(GL_TRIANGLES, 0, 6);
		glPopMatrix();

		glDisableClientState(GL_NORMAL_ARRAY);
		glDisableClientState(GL_COLOR_ARRAY);
		glDisable(GL_LIGHTING);
	glPopMatrix();	
}

- (void)layoutSubviews {
    [EAGLContext setCurrentContext:context];
    [self destroyFramebuffer];
    [self createFramebuffer];
    [self drawView];
}


- (BOOL)createFramebuffer {
    
    glGenFramebuffersOES(1, &viewFramebuffer);
    glGenRenderbuffersOES(1, &viewRenderbuffer);
    
    glBindFramebufferOES(GL_FRAMEBUFFER_OES, viewFramebuffer);
    glBindRenderbufferOES(GL_RENDERBUFFER_OES, viewRenderbuffer);
    [context renderbufferStorage:GL_RENDERBUFFER_OES fromDrawable:(CAEAGLLayer*)self.layer];
    glFramebufferRenderbufferOES(GL_FRAMEBUFFER_OES, GL_COLOR_ATTACHMENT0_OES, GL_RENDERBUFFER_OES, viewRenderbuffer);
    
    glGetRenderbufferParameterivOES(GL_RENDERBUFFER_OES, GL_RENDERBUFFER_WIDTH_OES, &backingWidth);
    glGetRenderbufferParameterivOES(GL_RENDERBUFFER_OES, GL_RENDERBUFFER_HEIGHT_OES, &backingHeight);
    
	glGenRenderbuffersOES(1, &depthRenderbuffer);
	glBindRenderbufferOES(GL_RENDERBUFFER_OES, depthRenderbuffer);
	glRenderbufferStorageOES(GL_RENDERBUFFER_OES, GL_DEPTH_COMPONENT16_OES, backingWidth, backingHeight);
	glFramebufferRenderbufferOES(GL_FRAMEBUFFER_OES, GL_DEPTH_ATTACHMENT_OES, GL_RENDERBUFFER_OES, depthRenderbuffer);
    
    if(glCheckFramebufferStatusOES(GL_FRAMEBUFFER_OES) != GL_FRAMEBUFFER_COMPLETE_OES) {
        NSLog(@"failed to make complete framebuffer object %x", glCheckFramebufferStatusOES(GL_FRAMEBUFFER_OES));
        return NO;
    }
    
    return YES;
}


- (void)destroyFramebuffer {
    
    glDeleteFramebuffersOES(1, &viewFramebuffer);
    viewFramebuffer = 0;
    glDeleteRenderbuffersOES(1, &viewRenderbuffer);
    viewRenderbuffer = 0;
    
    if(depthRenderbuffer) {
        glDeleteRenderbuffersOES(1, &depthRenderbuffer);
        depthRenderbuffer = 0;
    }
}


- (void)startAnimation {
    self.animationTimer = [NSTimer scheduledTimerWithTimeInterval:animationInterval target:self selector:@selector(drawView) userInfo:nil repeats:YES];
}


- (void)stopAnimation {
    self.animationTimer = nil;
}


- (void)setAnimationTimer:(NSTimer *)newTimer {
    [animationTimer invalidate];
    animationTimer = newTimer;
}


- (void)setAnimationInterval:(NSTimeInterval)interval {
    
    animationInterval = interval;
    if (animationTimer) {
        [self stopAnimation];
        [self startAnimation];
    }
}


- (void)dealloc {
    
    [self stopAnimation];
    
    if ([EAGLContext currentContext] == context) {
        [EAGLContext setCurrentContext:nil];
    }
    
    [context release];  
    [super dealloc];
}

- (void)audioPlayerDidFinishPlaying:(AVAudioPlayer *)player successfully:(BOOL)flag {
	NSLog(@"END");
	
	if ([[UIApplication sharedApplication] respondsToSelector:@selector(terminate)]) {
		[[UIApplication sharedApplication] performSelector:@selector(terminate)];
	} else {
		kill(getpid(), SIGINT); 
	}

}

-(UIImage *) glToUIImage {
    NSInteger myDataLength = 320 * 480 * 4;
   
    // allocate array and read pixels into it.
    GLubyte *buffer = (GLubyte *) malloc(myDataLength);
    glReadPixels(0, 0, 320, 480, GL_RGBA, GL_UNSIGNED_BYTE, buffer);
 
    // gl renders "upside down" so swap top to bottom into new array.
    // there's gotta be a better way, but this works.
    GLubyte *buffer2 = (GLubyte *) malloc(myDataLength);
    for(int y = 0; y <480; y++)
    {
        for(int x = 0; x <320 * 4; x++)
        {
            buffer2[(479 - y) * 320 * 4 + x] = buffer[y * 4 * 320 + x];
        }
    }
   
    // make data provider with data.
    CGDataProviderRef provider = CGDataProviderCreateWithData(NULL, buffer2, myDataLength, NULL);
   
    // prep the ingredients
    int bitsPerComponent = 8;
    int bitsPerPixel = 32;
    int bytesPerRow = 4 * 320;
    CGColorSpaceRef colorSpaceRef = CGColorSpaceCreateDeviceRGB();
    CGBitmapInfo bitmapInfo = kCGBitmapByteOrderDefault;
    CGColorRenderingIntent renderingIntent = kCGRenderingIntentDefault;
   
    // make the cgimage
    CGImageRef imageRef = CGImageCreate(320, 480, bitsPerComponent, bitsPerPixel, bytesPerRow, colorSpaceRef, bitmapInfo, provider, NULL, NO, renderingIntent);
   
    // then make the uiimage from that
    UIImage *myImage = [UIImage imageWithCGImage:imageRef];
    return myImage;
}
 
-(void)captureToPhotoAlbum {
    UIImage *image = [self glToUIImage];
    UIImageWriteToSavedPhotosAlbum(image, self, nil, nil); 
}

@end
